module Robojora
  class Robot
    NOT_IN_BOARD = 'Robot is not on the board'.freeze
    private_constant :NOT_IN_BOARD

    attr_reader :board, :name
    attr_accessor :facing

    def initialize(name)
      @name = name
      @board = board
    end

    def play_board(board)
      @board = board
    end

    def place(x_coordinate, y_coordinate, facing)
      raise ArgumentError, 'x_coordinate and y_coordinate must be integers' unless x_coordinate.is_a?(Integer) && y_coordinate.is_a?(Integer)
      raise CardinalDirectionError, 'facing should be a cardinal point string' unless Robojora::Settings.cardinal_points.include?(facing) || !select_square_by_name.nil?

      square = select_square_by_coordinates(x_coordinate, y_coordinate)
      robot_removing_and_placing(facing, square) if square
    end

    def report
      select_square_by_name && facing ? location_report : NOT_IN_BOARD
    end

    def right
      self.facing = rotator.right
    end

    def left
      self.facing = rotator.left
    end

    def move
      raise Robojora::RobotNotOnBoardError if not_movable?

      mover.move
    end

    def select_square_by_name
      board.select { |square| square.presence == name }.first
    end

    def select_square_by_coordinates(x_coordinate, y_coordinate)
      board.select { |square| square.coordinates == [x_coordinate, y_coordinate] }.first
    end

    def robot_removing_and_placing(facing, square)
      remove_robot_from_current_square
      self.facing = facing
      square.presence = name
      true
    end

    private

    def location_report
      "#{select_square_by_name.coordinates.join(', ')}, #{facing}"
    end

    def remove_robot_from_current_square
      current_square = select_square_by_name
      current_square.presence = nil if current_square
    end

    def rotator
      Robojora::Rotator.new(facing)
    end

    def mover
      Robojora::Movers.const_get(facing.capitalize).new(self)
    end

    def not_movable?
      select_square_by_name.nil? || select_square_by_name.borders.include?(facing)
    end
  end
end
